import * as dotenv from "dotenv";
import path from "node:path";
import { downloadImagefromURL, uploadToGhost } from "./images";
import { Convertor } from "./Convertor";

// Init config
dotenv.config();

export interface StackedImage {
  path: string;
  url: string;
}

export class ImageProcessor {
  images: Record<string, StackedImage> = {};

  /**
   * Identifies images and tries to process them
   * @param api the ghost api reference
   * @param basepath the path where to save images hosted online
   */
  public async processImages(processor: Convertor, api: any, basepath: string) {
    // collect images in an array
    const imageRegex = /!\[[^\]]*\]\((.*?)\s*("(?:.*[^"])")?\s*\)/g;
    //let images: string[] = [];
    //let m: any;
    [
      ...processor.content.matchAll(imageRegex),
      ["", "@@@" + processor.metadata.cover, ""],
    ].forEach(async (item) => {
      console.log(JSON.stringify(item));
      if (item[1] in this.images) {
        console.log("Skip cached image for URL: ", item[1]);
      } else {
        let url = item[1];
        // Add the image to the cache of processed images
        this.images[url] = {
          path: url,
          url: url,
        };

        let localpath = url;

        if (url.startsWith("@@@")) {
          localpath = "" + process.env.HEAD_IMAGE_PATH + url.slice(3);
        } else {
          localpath = path.join(path.dirname(processor.filename), url);
        }

        // Download images locally if not already
        if (url.toLowerCase().startsWith("http")) {
          // Download the image locally
          let localpath = path.join(basepath, path.basename(url));
          await downloadImagefromURL(url, localpath);
        }

        // Upload the image to ghost
        if (api) {
          let reference = await uploadToGhost(api, localpath);
          this.images[url].url = reference.url;
        }

        // replace the image string with the new URL
        console.log(
          "replace",
          this.images[url].path,
          " with ",
          this.images[url].url
        );
        processor.content = processor.content.replaceAll(
          this.images[url].path,
          this.images[url].url
        );

        // Set the feature image if any
        if (this.images[url].path.startsWith("@@@")) {
          processor.setFeatureImage(this.images[url].url);
        }
      }

      // debug: show the updated content if necessary
      // console.log(this.content);
    });
  }
}
