import fs from "fs";
import http from "http";
import https from "https";
import path from "path";
import { Blob } from "buffer";

import stream from "stream";
//import { timer } from "./timer";

const Stream = stream.Transform;

/**
 * Download an image off an URL
 * @param url the url containing the image
 * @param filename the filename to be saved
 */
export async function downloadImagefromURL(url: string, filename: string) {
  let client: any = http;
  let result = 200;

  if (url.toString().indexOf("https") === 0) {
    client = https;
  }

  client
    .request(url, function (response: any) {
      var data = new Stream();
      result = response.statusCode;

      response.on("data", function (chunk: any) {
        data.push(chunk);
      });
      response.on("end", function () {
        fs.writeFileSync(filename, data.read());
      });
    })
    .end();
  return result;
}

interface Image {
  file: Blob | File | string;
  purpose: "image" | "profile_image" | "icon";
  ref?: string;
}

export interface GhostReference {
  url: string;
  ref: string;
}

/**
 *
 * @param api the Ghost API object
 * @param filename the file name of the image to be uploaded
 * @returns a GhostReference promise with the real file name.
 */
export async function uploadToGhost(
  api: any,
  filename: string
): Promise<GhostReference> {
  let image: Image = {
    ref: path.basename(filename),
    file: filename,
    purpose: "image",
  };
  //console.log("Upload: [%s]", filename);
  // await timer(500 * 1);

  let retries = 0;
  let pass = true;
  let result: any;
  while (retries < 5 && pass) {
    await api.images
      .upload(image)
      .then((r: any) => {
        result = r;
        pass = false;
      })
      .catch((r: any) => {
        console.log(JSON.stringify(r));
        retries++;
      });
  }
  return result;
}
